# -*- coding: UTF-8 -*-
"""
Parseur DOM des fichiers XML de collecte des variables eoles
"""

import xml.dom.minidom 
from creole.error import *
from creole.utils import string_to_bool, get_text_node
     

def parse_xml_file(filename):
    """
    @param filename: nom du fichier xml source
    @return: structure de données permettant de créer les objets Eole
    """
    try:
        document = xml.dom.minidom.parse(filename)
        return _parse_root_node(document)
    except Exception, e:
        raise ConfigError, "Erreur lors du parsing du fichier %s : %s" % (filename, str(e))

def parse_string(string):
    try:
        document = xml.dom.minidom.parseString(string)
        return _parse_root_node(document)
    except Exception, e:
        raise ConfigError, "Erreur lors du parsing : %s" % str(e)

def _parse_root_node(document):
    """
    @param document: le document DOM contenant l'arborescence xml
    """
    root_node = document.documentElement


    variables = parse_variables(root_node.getElementsByTagName('variables')[0])
    families, order = parse_families(root_node.getElementsByTagName('variables')[0])
    file_node = root_node.getElementsByTagName('files')
    if file_node != []:
        files = parse_files(file_node[0])
    else:
        files = {}
    constraint_node = root_node.getElementsByTagName('constraints')
    if constraint_node != []:
        constraints = parse_constraints(constraint_node[0])
    else:
        constraints = {'checks' : {},
                       'fills' : {},
                       'conditions' : {},
                       'auto' : {}
                      }
    group_node = root_node.getElementsByTagName('constraints')
    if group_node != []:
        groups = parse_groups(group_node[0])
    else:
        groups = {}
    help_node = root_node.getElementsByTagName('help')
    if help_node != []:
        helps = parse_help(help_node[0])
    else:
        helps = {'variables':{},
                 'families':{}
                }
    sep_node = root_node.getElementsByTagName('separators')
    if sep_node != []:
        separators = parse_separators(sep_node[0])
    else:
        separators = {}
 
    return files, variables, families, order, constraints, groups, helps, separators


def parse_files(file_node):
    result = {}
    for file in file_node.getElementsByTagName('file'):
        result[file.getAttribute('name')] = {'hidden':False, 
                                             'source':file.getAttribute('source'),
                                             'mode':file.getAttribute('mode'),
                                             'owner':file.getAttribute('owner'),
                                             'group':file.getAttribute('group'),
                                             'filelist':file.getAttribute('filelist'),
                                            }
    return result
    

def parse_groups(node):
    result = {}
    for group in node.getElementsByTagName('group'):
        slaves = [get_text_node(slave)
                  for slave in group.getElementsByTagName('slave')]
        result[group.getAttribute('master')] = slaves
    return result
    

def parse_variables(var_node):
    """
    @param var_node: node DOM des variables
    """
    result = {}
    for var in var_node.getElementsByTagName('variable'):
        try:
            hidden = string_to_bool(var.getAttribute('hidden'))
        except (TypeError, ValueError):
            hidden = False
        try:
            multi = string_to_bool(var.getAttribute('multi'))
        except (TypeError, ValueError):
            multi = False
        name = var.getAttribute('name')
        result[name] = dict(value=parse_value(var),
                            type=var.getAttribute('type'),
                            description=var.getAttribute('description'),
                            hidden=hidden,
                            multi=multi,
                            auto=var.getAttribute('auto')
                            )
    return result


def parse_constraints(node):
    """@param node: node DOM des contraintes"""
    constraints = {'checks' : parse_funcs(node,'check'),
                   'fills' : parse_funcs(node,'fill'),
                   'auto' : parse_funcs(node,'auto'),
                   'conditions' : parse_conditions(node)
                  }
    return constraints
    

def parse_funcs(node, func_type):
    """
    @param node: node DOM des fonctions
    @param func_type: TagName of the functions to find
    """
    # fonctions de vérification
    funcs = {}
    for func in node.getElementsByTagName(func_type):
        # lecture des paramètres
        # XXX (adim): est-ce que le fait que params ne soit pas réinitialisé
        #             à chaque tour de boucle "for target in targets" est un bug ?
        params = []
        targets = [ get_text_node(t) for t in func.getElementsByTagName('target') ]
        if not targets:
            targets = [func.getAttribute('target')]
        for target in targets:
            if target is not None:
                for param in func.getElementsByTagName('param'):
                    params.append(parse_param(param))
                funcs.setdefault(target, []).append([func.getAttribute('name'),
                                                     params])
    return funcs

def parse_conditions(node):
    """
    @param node: node DOM des fonctions
    """
    # fonctions de vérification
    funcs = {}
    for func in node.getElementsByTagName('condition'):
        # lecture des paramètres
        targets = []
        family_targets = []
        file_targets = []
        filelist_targets = []
        # paramètres de la fonction
        params = [parse_param(param)
                  for param in func.getElementsByTagName('param')]
        # cibles de la dépendance
        for target in func.getElementsByTagName('target'):
            type = target.getAttribute('type')
            if type == 'family':
                family_targets.append(get_text_node(target))
            elif type == 'file':
                file_targets.append(get_text_node(target))
            elif type == 'filelist':
                filelist_targets.append(get_text_node(target))
            else:
                targets.append(get_text_node(target))
        funcdef = [func.getAttribute('name'), family_targets, targets,
                   file_targets, filelist_targets, params]
        funcs.setdefault(func.getAttribute('source'), []).append(funcdef)
    return funcs


def parse_help(node):
    var_help = dict( (var.getAttribute('name'), get_text_node(var).strip())
                     for var in node.getElementsByTagName('variable') )
    fam_help = dict( (var.getAttribute('name'), get_text_node(var).strip())
                     for var in node.getElementsByTagName('family') )
    return {'variables':var_help, 'families': fam_help}

def parse_separators(node):
    """dictionnaire des séparateurs, format {'variable':'text'}
    variable : nom de la première variable après le sépateur"""
    var_sep = {}
    for var in node.getElementsByTagName('separator'):
        var_sep[var.getAttribute('name')] = get_text_node(var).strip()
    return var_sep
 
def parse_param(param_node):
    return {'name'  : param_node.getAttribute('name'),
            'type'  : param_node.getAttribute('type'),
            'value' : get_text_node(param_node)
            }

def parse_value(varnode):
    return [get_text_node(val) for val in varnode.getElementsByTagName('value')]
 
def parse_families(var_node):
    result = {}
    order = []
    families = var_node.getElementsByTagName('family')
    for family in families:
        family_name = family.getAttribute('name')
        order.append(family_name)
        hidden = family.getAttribute('hidden')
        mode = family.getAttribute('mode')
        if hidden == 'True':
            hidden = True
        else:
            hidden = False
        result[family_name] = {'hidden':hidden, 'mode':mode, 'vars':get_vars_name(family)}
    return result, order


def get_vars_name(family_node):
    return [var.getAttribute('name')
            for var in family_node.getElementsByTagName('variable')]
