# -*- coding: UTF-8 -*-
import os,sys,re,string,glob
from error import TypeEoleError, ValueEoleError, DependencyError
from creole.eosautofunc import *

# mots clés gérés : obligatoire, option, ip, network, netmask, booleen, nombre
# le dictionnaire suivant décrit tous les mot-clés gérés, et les fonctions de validation qui leur  sont associés

classes = {
    '128.0.0.0' : '1'
    , '192.0.0.0' : '2'
    , '224.0.0.0' : '3'
    , '240.0.0.0' : '4'
    , '248.0.0.0' : '5'
    , '252.0.0.0' : '6'
    , '254.0.0.0' : '7'
    , '255.0.0.0' : '8'
    , '255.128.0.0' : '9'
    , '255.192.0.0' : '10'
    , '255.224.0.0' : '11'
    , '255.240.0.0' : '12'
    , '255.248.0.0' : '13'
    , '255.252.0.0' : '14'
    , '255.254.0.0' : '15'
    , '255.255.0.0' : '16'
    , '255.255.128.0' : '17'
    , '255.255.192.0' : '18'
    , '255.255.224.0' : '19'
    , '255.255.240.0' : '20'
    , '255.255.248.0' : '21'
    , '255.255.252.0' : '22'
    , '255.255.254.0' : '23'
    , '255.255.255.0' : '24'
    , '255.255.255.128' : '25'
    , '255.255.255.192' : '26'
    , '255.255.255.224' : '27'
    , '255.255.255.240' : '28'
    , '255.255.255.248' : '29'
    , '255.255.255.252' : '30'
    , '255.255.255.254' : '31'
    , '255.255.255.255' : '32'
}

# type paramètre

class Param:
    """paramètre pour les fonctions de contrainte"""
    def __init__(self,value,name=None,type=None):
        self.value = value
        self.name = name
        self.type = type


#############################
## Fonctions de vérification
#############################

def valid_ip(data):
    """fonction de validation de la saisie d'une adresse ip"""
    
    # variable non obligatoire
    if data == "":
        return True
    
    # découpage de l'ip
    ip = data.split('.')
    if len(ip) != 4:
        raise TypeEoleError("mauvaise syntaxe d'adresse ip")
    
    # vérification des nombres
    for part in ip:
        try:
            nb = int(part)
        except ValueError:
            raise TypeEoleError("l'ip doit être constituée de 4 nombres séparés par des points")
        else:
            if not (0 <= nb < 256):
                raise TypeEoleError("les nombres doivent etre compris entre 0 et 255")
        
    # on n'a pas rencontré d'erreur        
    return True

def valid_intervalle(data):
    """ fonction de validation d'une plage d'ip """
    if data == "":
        return True
    
    liste = data.split()
    if len(liste) != 2:
        raise TypeEoleError("vous devez spécifier 2 adresses ip")
    else:
        # on vérifie la syntaxe des ips
        for ip in liste:
            valid_ip(ip)
    return True

def valid_network(data,ip=None):
    """fonction de validation de la saisie d'un réseau"""
    if data == "":
        return True
    
    # on utilise la fonction de test d'adresse ip pour la syntaxe
    valid_ip(data)
    if ip is not None:
        # on effectue les tests spécifiques au réseau (fonction de l'ip ?)
        ip_parts = ip.split(".")
        network_parts = data.split(".")
        for i in range(4):
            if (int(network_parts[i]) != 0) and (network_parts[i] != ip_parts[i]):
                raise TypeEoleError("le réseau spécifié ne correspond pas à l'ip "+ip)
    
    # aucune erreur rencontrée
    return True

def valid_netmask(data):
    """fonction de validation de la saisie d'un masque réseau"""
    if data == "":
        return True
    
    # on utilise la fonction de test d'adresse ip pour la syntaxe
    valid_ip(data)
    # on effectue les tests spécifiques aux masques réseaux
    netmask = []
    for part in data.split("."):
        netmask.append(int(part))
    error = 0
    mask_len=0
    done = 0
    for part in netmask:
        bitmask = 0x80
        for bit in range(8):
            if not done:
                if part & bitmask:
                    mask_len = mask_len + 1
                else:
                    done = 1
            else:
                if part & bitmask:
                    error = 1
            bitmask = bitmask >> 1

    if error == 1:
        raise TypeEoleError("syntaxe de masque réseau invalide")
    # pas d'erreur rencontrée
    return True

def valid_booleen(data,type_bool="alpha"):
    """fonction de validation d'une réponse booléenne
       type_bool : alpha ou num (O/N ou 0/1)
    """
    if data == "":
        return True

    if (data[0].upper() != 'O') and (data[0].upper() != 'N'):
        raise TypeEoleError("vous devez repondre par 'O' ou 'N'")
    # pas d'erreurs détectées
    return True

def valid_enum(data,liste=['oui','non','Oui','Non','OUI','NON','o','n','O','N'],checkval="True"):
    """fonction de validation d'une réponse en fonction
    d'une liste de valeur possibles
    """
    if data == "":
        return True

    try:
        if type(liste) != list:
            liste = eval(liste)
    except:
        raise TypeEoleError("la liste des valeurs possibles n'est pas valide")
    if data not in liste and checkval != "False":
        raise TypeEoleError("repondez par un des choix suivants : "+str(liste))
    return True

def valid_regexp(data,*args):
    """fonction de validation d'une saisie par expression régulière"""
    # construction de la chaine d'expression régulière
    if data == "":
        return True

    exp_reg = ""
    for arg in args:
        exp_reg += arg
        
    m=re.match(exp_reg,data)
    if m is None:
        raise TypeEoleError("la syntaxe est incorrecte")
    else:
        return True
     

def valid_entier(data,mini=None,maxi=None):
    """ fonction de validation d'un nombre (intervalle optionnel)"""
    if data == "":
        return True
    try:
        value = int(data)
    except ValueError:
        raise TypeEoleError("vous devez saisir un nombre")
    else:
        if mini is not None and value < int(mini):
            raise TypeEoleError("donnez un entier supérieur à "+str(mini))
        if maxi is not None and value > int(maxi):
            raise TypeEoleError("donnez un entier inférieur à "+str(maxi))
        return True


def valid_yesno(data):
    """Yes ou No pour les fichiers de config"""
    if data not in ["Yes","No"]:
        raise TypeEoleError("la valeur doit être Yes ou No")
    return True


def valid_system(data,cmd,result,*args):
    if data == "":
        return True
    
    cmd_string =  cmd
    for arg in args:
        cmd_string += " "+arg
    
    # exécution de la fonction
    code_retour = os.system(cmd_string)
    
    if code_retour == result:
        return True
    else:
        raise TypeEoleError("la commande "+cmd_string+" a echoué")


def valid_differ(data,value):
    if data == value:
        raise TypeEoleError("la valeur doit être différente de %s"%value)
    else:
        return True


def valid_chaine(data, forbidden_chars=[' ']):
    """Recherche de caractèresinterdits dans une chaine"""
    forbidden_chars = eval(forbidden_chars)
    for char in forbidden_chars:
        if char in data:
            raise TypeEoleError("les caractères (%s) sont interdits"%','.join(forbidden_chars))
    return True


def obligatoire(data):
    """ fonction qui vérifie que les données ne sont pas vides
    """
    if data == [] or data == "" or data[0].strip() == "":
        raise TypeEoleError("ce paramètre est obligatoire")
    else:
        return True

################################################################
## Fonctions de calcul de dépendance entre variables et groupes 
################################################################

def hidden_if_not_in(val, *args):
    if val not in args:
        raise DependencyError("""%s n'est pas dans %s""" % (val, args))

def hidden_if_in(val, *args):
    if val in args:
        raise DependencyError("""%s est dans %s""" % (val, args))

# TODO : fonctions du genre range() ??

#############################################
## Fonctions de saisie et calcul des valeurs
#############################################

def calc_network(ip, netmask):
    """ calcule le réseau par défaut à partir d'un masque et d'une ip """
    # si les ip et netmask viennent du dictionnaire, on les récupère
    if ip == "" or netmask == "":
        return ""
    broadcast, network = calc_adresses(ip, netmask)
            
    return network

def calc_broadcast(ip, netmask):
    """ calcule l'adresse de broadcast par défaut à partir d'un masque et d'une ip """
    # si les ip et netmask viennent du dictionnaire, on les récupère
    if ip == "" or netmask == "":
        return ""
    broadcast, network = calc_adresses(ip, netmask)
            
    return broadcast

def create_ip(ip_list):
    template = """%i.%i.%i.%i"""
    return template%tuple(ip_list)
    
def create_ip_list(ip):
    a,b,c,d=ip.split('.')
    return [int(a),int(b),int(c),int(d)]

def calc_adresses(ip, mask):
    """calcul l'adresse de broadcast à partir de l'ip et du netmask
    """
    mask_list = create_ip_list(mask)
    ip_list = create_ip_list(ip)
    broadcast_list = []
    network_list = []
    for i in range(4):
        #pour l'adresse réseau c'est simple
        network_list.append(int(ip_list[i]) & int(mask_list[i]))
        #broadcast_part=255 si mask[i]=255
        for j in range(256):
            if mask_list[i]&j == 0:
                broadcast_part = j
        #calcul du masque spécial
        if broadcast_part != 255:
            broadcast_part += mask_list[i]&ip_list[i]
        broadcast_list.append(broadcast_part)
    broadcast = create_ip(broadcast_list)
    network = create_ip(network_list)
    return broadcast, network

def calc_classe(netmask):
    """calcule la classe du réseau à partir du netmask"""

    if classes.has_key(netmask):
        return(classes[netmask])
    else:
        return(netmask)

def calc_netmask(plage):
    """calcule le reseau a partir de sa classe"""
    if plage in classes.values():
        for mask, plaj in classes.items():
            if plaj == plage:
                return mask
    else:
        return plage


def calc_pg_cache_size():
    """
    Calcul la taille du cache postgreql en fonction de la RAM
    """
    procmem = open('/proc/meminfo')
    meminfo = procmem.read().split('\n')
    procmem.close()
    for line in meminfo:
        if line.startswith('MemTotal'):
            mem_tot = float(line.split()[1])
            break
    mem_cache = int( (mem_tot * 0.72) / 8 )
    return str(mem_cache)

def calc_val(valeur):
    return(valeur)

####################################################
# fonctions diverses accessibles dans les templates
####################################################

def list_files(rep, wildcards='*', stripext=False, basename=True, default=[]):
    res = default
    if os.path.isdir(rep):
        res = []
        for fic in glob.glob(rep+'/%s' % wildcards):
            if basename:
                fic = os.path.basename(fic)
            if stripext:
                fic = os.path.splitext(fic)[0]
            res.append(fic)
        res.sort()
    return res

def is_file(filename):
    """filtre permettant de verifier l'existence d'un fichier avant de l'inclure par exemple """
    return os.path.isfile(filename)

def list(lst, sep=' '):
    """retourne une liste concatenee de donnees de la liste"""
    return sep.join(lst)

def purge_list(lst):
    """supprime les doublons dans une liste"""
    res = []
    for elt in lst:
        if elt not in res:
            res.append(elt)
    return res

def main():
    """fonction de test  de la bibliothèque"""
    resultat,msg = valid_netmask(0,sys.argv[1]) 
    print str(resultat)+", message : "+msg+"!"

if __name__ == "__main__":
    main()
